@@ -2,6 +2,7 @@
 import base64
 import hashlib
 import re
+from decimal import Decimal
 
 from itertools import product
 
@@ -24,7 +25,7 @@ from qr_code.tests import (
     TEST_TEXT,
     OVERRIDE_CACHES_SETTING,
     COMPLEX_TEST_TEXT,
-    BASE64_PNG_IMAGE_TEMPLATE,
+    get_base64_png_image_template,
     IMAGE_TAG_BASE64_DATA_RE,
 )
 from qr_code.tests.utils import (
@@ -45,6 +46,9 @@ class TestQRUrlFromTextResult(SimpleTestCase):
 
     default_ref_base_file_name = "qrfromtext_default"
 
+    def setUp(self):
+        self.maxDiff = None
+
     @staticmethod
     def _get_reference_result_for_default_svg():
         return get_svg_content_from_file_name(TestQRUrlFromTextResult.default_ref_base_file_name)
@@ -78,7 +82,7 @@ class TestQRUrlFromTextResult(SimpleTestCase):
             url3 = qr_url_from_text(TEST_TEXT, image_format="svg", size=1, **url_options_kwargs)
             url4 = qr_url_from_text(TEST_TEXT, image_format="SVG", size=1, **url_options_kwargs)
             url5 = qr_url_from_text(TEST_TEXT, options=QRCodeOptions(image_format="SVG", size=1), **url_options_kwargs)
-            # Using an invalid image format should fallback to SVG.
+            # Using an invalid image format should fall back to SVG.
             url6 = qr_url_from_text(TEST_TEXT, image_format="invalid-format-name", size=1, **url_options_kwargs)
             url = url1
             if url_signature_enabled is not False:
@@ -544,10 +548,24 @@ class TestQRFromTextSvgResult(SimpleTestCase):
     The tests cover direct call to tag function, rendering of tag, and direct call to qr_code API.
     """
 
+    def setUp(self):
+        self.maxDiff = None
+
     def test_size(self):
         base_ref_file_name = "qrfromtext_size"
-        sizes = ["t", "T", "s", "S", None, -1, 0, "m", "M", "l", "L", "h", "H", "6", 6, "8", 8, "10", 10]
-        size_names = ["tiny"] * 2 + ["small"] * 2 + ["medium"] * 5 + ["large"] * 2 + ["huge"] * 2 + ["6"] * 2 + ["8"] * 2 + ["10"] * 2
+        sizes = ["t", "T", "s", "S", None, -1, 0, "m", "M", "l", "L", "h", "H", "6", 6, "8", 8, "10", 10, 2.1, Decimal("1.2")]
+        size_names = (
+            ["tiny"] * 2
+            + ["small"] * 2
+            + ["medium"] * 5
+            + ["large"] * 2
+            + ["huge"] * 2
+            + ["6"] * 2
+            + ["8"] * 2
+            + ["10"] * 2
+            + ["2.1"]
+            + ["1.2"]
+        )
         for i in range(len(sizes)):
             size = sizes[i]
             print("Testing SVG with size %s" % size)
@@ -721,6 +739,86 @@ class TestQRFromTextSvgResult(SimpleTestCase):
             ref_image_data = get_svg_content_from_file_name(test_data["ref_file_name"])
             self.assertEqual(minimal_svg(source_image_data), minimal_svg(ref_image_data))
 
+    def test_colors(self):
+        def run_tests():
+            if REFRESH_REFERENCE_IMAGES:
+                write_svg_content_to_file(result_file_name, qr1)
+            self.assertEqual(minimal_svg(qr1), minimal_svg(qr2))
+            self.assertEqual(minimal_svg(qr1), minimal_svg(qr3))
+            ref_image_data = minimal_svg(get_svg_content_from_file_name(result_file_name))
+            self.assertEqual(minimal_svg(qr1), ref_image_data)
+
+        base_ref_file_name = "qrfromtext_color"
+        colors = ["#000", "#000000", "black", "#fff", "#FFFFFF", "white", "red", "blue", None]
+        color_names = ["black"] * 2 + ["black"] + ["white_hexa"] * 2 + ["white"] + ["red", "blue", "transparent"]
+        for i in range(len(colors)):
+            color = colors[i]
+            color_name = color_names[i]
+            print("Testing SVG with dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_dark_{color_name}"
+            # Set light color to pink  to avoid black on black or white on white.
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(dark_color=color, light_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, dark_color=color, light_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(dark_color=color, light_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_light_{color_name}"
+            # Set dark color to pink  to avoid black on black or white on white.
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(light_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, light_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(light_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with data_dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_data_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(data_dark_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, data_dark_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(data_dark_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with data light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_data_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(data_light_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, data_light_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(data_light_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with finder dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_finder_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(finder_dark_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, finder_dark_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(finder_dark_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with finder light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_finder_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(finder_light_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, finder_light_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(finder_light_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with dark module color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_dark_module_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(dark_module_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, dark_module_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(dark_module_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with alignment dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_alignment_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(alignment_dark_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, alignment_dark_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(alignment_dark_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
+            print("Testing SVG with alignment light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_alignment_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(alignment_light_color=color, dark_color="pink", image_format="svg"))
+            qr2 = qr_from_text(TEST_TEXT, alignment_light_color=color, dark_color="pink", image_format="svg")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(alignment_light_color=color, dark_color="pink", image_format="svg"))
+            run_tests()
+
 
 class TestQRFromTextPngResult(SimpleTestCase):
     """
@@ -729,10 +827,15 @@ class TestQRFromTextPngResult(SimpleTestCase):
     The tests cover direct call to tag function, rendering of tag, and direct call to qr_code API.
     """
 
+    def setUp(self):
+        self.maxDiff = None
+
     def test_size(self):
         base_ref_file_name = "qrfromtext_size"
-        sizes = ["t", "T", "s", "S", None, -1, 0, "m", "M", "l", "L", "h", "H", "6", 6, "8", 8, "10", 10]
-        size_names = ["tiny"] * 2 + ["small"] * 2 + ["medium"] * 5 + ["large"] * 2 + ["huge"] * 2 + ["6"] * 2 + ["8"] * 2 + ["10"] * 2
+        sizes = ["t", "T", "s", "S", None, -1, 0, "m", "M", "l", "L", "h", "H", "6", 6, "8", 8, "10", 10, 2.1]
+        size_names = (
+            ["tiny"] * 2 + ["small"] * 2 + ["medium"] * 5 + ["large"] * 2 + ["huge"] * 2 + ["6"] * 2 + ["8"] * 2 + ["10"] * 2 + ["2.1"]
+        )
         for i in range(len(sizes)):
             size = sizes[i]
             print("Testing PNG with size %s" % size)
@@ -748,7 +851,7 @@ class TestQRFromTextPngResult(SimpleTestCase):
             result = base64.b64encode(get_png_content_from_file_name(result_file_name)).decode("utf-8")
             self.assertEqual(qr1, qr2)
             self.assertEqual(qr1, qr3)
-            self.assertEqual(qr1, BASE64_PNG_IMAGE_TEMPLATE % result)
+            self.assertEqual(qr1, get_base64_png_image_template() % result)
 
     def test_version(self):
         base_file_name = "qrfromtext_version"
@@ -771,7 +874,7 @@ class TestQRFromTextPngResult(SimpleTestCase):
             self.assertEqual(qr1, qr2)
             self.assertEqual(qr1, qr3)
             self.assertEqual(qr1, qr4)
-            self.assertEqual(qr1, BASE64_PNG_IMAGE_TEMPLATE % result)
+            self.assertEqual(qr1, get_base64_png_image_template() % result)
 
     def test_error_correction(self):
         file_base_name = "qrfromtext_error_correction"
@@ -924,3 +1027,83 @@ class TestQRFromTextPngResult(SimpleTestCase):
                 write_png_content_to_file(test_data["ref_file_name"], source_image_data)
             ref_image_data = get_png_content_from_file_name(test_data["ref_file_name"])
             self.assertEqual(source_image_data, ref_image_data)
+
+    def test_colors(self):
+        def run_tests():
+            if REFRESH_REFERENCE_IMAGES:
+                match = IMAGE_TAG_BASE64_DATA_RE.search(qr1)
+                source_image_data = match.group("data")
+                write_png_content_to_file(result_file_name, base64.b64decode(source_image_data))
+            result = base64.b64encode(get_png_content_from_file_name(result_file_name)).decode("utf-8")
+            self.assertEqual(qr1, qr2)
+            self.assertEqual(qr1, qr3)
+            self.assertEqual(qr1, get_base64_png_image_template() % result)
+
+        base_ref_file_name = "qrfromtext_color"
+        colors = ["#000", "#000000", "black", "#fff", "#FFFFFF", "white", "red", "blue", None]
+        color_names = ["black"] * 2 + ["black"] + ["white_hexa"] * 2 + ["white"] + ["red", "blue", "transparent"]
+        for i in range(len(colors)):
+            color = colors[i]
+            color_name = color_names[i]
+            print("Testing PNG with dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(dark_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, dark_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(dark_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(light_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, light_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(light_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with data_dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_data_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(data_dark_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, data_dark_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(data_dark_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with data light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_data_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(data_light_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, data_light_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(data_light_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with finder dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_finder_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(finder_dark_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, finder_dark_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(finder_dark_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with finder light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_finder_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(finder_light_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, finder_light_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(finder_light_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with dark module color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_dark_module_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(dark_module_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, dark_module_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(dark_module_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with alignment dark color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_alignment_dark_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(alignment_dark_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, alignment_dark_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(alignment_dark_color=color, image_format="png"))
+            run_tests()
+
+            print("Testing PNG with alignment light color %s" % color_name)
+            result_file_name = f"{base_ref_file_name}_alignment_light_{color_name}"
+            qr1 = make_embedded_qr_code(TEST_TEXT, QRCodeOptions(alignment_light_color=color, image_format="png"))
+            qr2 = qr_from_text(TEST_TEXT, alignment_light_color=color, image_format="png")
+            qr3 = qr_from_text(TEST_TEXT, options=QRCodeOptions(alignment_light_color=color, image_format="png"))
+            run_tests()
